@@ -21,14 +21,13 @@
 import datetime
 import itertools
 import json
-import uuid
 
 import croniter
 from oslo_config import cfg
-from oslo_context import context
 from oslo_log import log
 from oslo_utils import netutils
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 import pecan
 from pecan import rest
 import pytz
@@ -40,7 +39,6 @@ from wsme import types as wtypes
 import wsmeext.pecan as wsme_pecan
 
 import aodh
-from aodh.api.controllers.v2.alarm_rules import combination
 from aodh.api.controllers.v2 import base
 from aodh.api.controllers.v2 import utils as v2_utils
 from aodh.api import rbac
@@ -49,25 +47,22 @@ from aodh import keystone_client
 from aodh import messaging
 from aodh import notifier
 from aodh.storage import models
-from aodh import utils
 
 LOG = log.getLogger(__name__)
 
 
 ALARM_API_OPTS = [
     cfg.IntOpt('user_alarm_quota',
-               default=None,
-               deprecated_group="alarm",
+               deprecated_group='DEFAULT',
                help='Maximum number of alarms defined for a user.'
                ),
     cfg.IntOpt('project_alarm_quota',
-               default=None,
-               deprecated_group="alarm",
+               deprecated_group='DEFAULT',
                help='Maximum number of alarms defined for a project.'
                ),
     cfg.IntOpt('alarm_max_actions',
                default=-1,
-               deprecated_group="alarm",
+               deprecated_group='DEFAULT',
                help='Maximum count of actions for each state of an alarm, '
                     'non-positive number means no limit.'),
 ]
@@ -77,6 +72,9 @@ state_kind_enum = wtypes.Enum(str, *state_kind)
 severity_kind = ["low", "moderate", "critical"]
 severity_kind_enum = wtypes.Enum(str, *severity_kind)
 
+ALARM_REASON_DEFAULT = "Not evaluated yet"
+ALARM_REASON_MANUAL = "Manually set via API"
+
 
 class OverQuota(base.ClientSideError):
     def __init__(self, data):
@@ -100,14 +98,14 @@ def is_over_quota(conn, project_id, user_id):
     over_quota = False
 
     # Start by checking for user quota
-    user_alarm_quota = pecan.request.cfg.user_alarm_quota
+    user_alarm_quota = pecan.request.cfg.api.user_alarm_quota
     if user_alarm_quota is not None:
         user_alarms = list(conn.get_alarms(user=user_id))
         over_quota = len(user_alarms) >= user_alarm_quota
 
     # If the user quota isn't reached, we check for the project quota
     if not over_quota:
-        project_alarm_quota = pecan.request.cfg.project_alarm_quota
+        project_alarm_quota = pecan.request.cfg.api.project_alarm_quota
         if project_alarm_quota is not None:
             project_alarms = list(conn.get_alarms(project=project_id))
             over_quota = len(project_alarms) >= project_alarm_quota
@@ -188,13 +186,7 @@ ACTIONS_SCHEMA = extension.ExtensionManager(
 
 
 class Alarm(base.Base):
-    """Representation of an alarm.
-
-    .. note::
-        combination_rule and threshold_rule are mutually exclusive. The *type*
-        of the alarm should be set to *threshold* or *combination* and the
-        appropriate rule should be filled.
-    """
+    """Representation of an alarm."""
 
     alarm_id = wtypes.text
     "The UUID of the alarm"
@@ -259,6 +251,9 @@ class Alarm(base.Base):
     state_timestamp = datetime.datetime
     "The date of the last alarm state changed"
 
+    state_reason = wsme.wsattr(wtypes.text, default=ALARM_REASON_DEFAULT)
+    "The reason of the current state"
+
     severity = base.AdvEnum('severity', str, *severity_kind,
                             default='low')
     "The severity of the alarm"
@@ -274,9 +269,21 @@ class Alarm(base.Base):
             self.time_constraints = [AlarmTimeConstraint(**tc)
                                      for tc in time_constraints]
 
+    @classmethod
+    def from_db_model_scrubbed(cls, m):
+        # Return an Alarm from a DB model with trust IDs scrubbed from actions
+        data = m.as_dict()
+
+        for field in ('ok_actions', 'alarm_actions',
+                      'insufficient_data_actions'):
+            if data.get(field) is not None:
+                data[field] = [cls._scrub_action_url(action)
+                               for action in data[field]]
+
+        return cls(**data)
+
     @staticmethod
     def validate(alarm):
-
         Alarm.check_rule(alarm)
         Alarm.check_alarm_actions(alarm)
 
@@ -312,7 +319,7 @@ class Alarm(base.Base):
 
     @staticmethod
     def check_alarm_actions(alarm):
-        max_actions = pecan.request.cfg.alarm_max_actions
+        max_actions = pecan.request.cfg.api.alarm_max_actions
         for state in state_kind:
             actions_name = state.replace(" ", "_") + '_actions'
             actions = getattr(alarm, actions_name)
@@ -321,8 +328,8 @@ class Alarm(base.Base):
 
             action_set = set(actions)
             if len(actions) != len(action_set):
-                LOG.info(_('duplicate actions are found: %s, '
-                           'remove duplicate ones') % actions)
+                LOG.info('duplicate actions are found: %s, '
+                         'remove duplicate ones', actions)
                 actions = list(action_set)
                 setattr(alarm, actions_name, actions)
 
@@ -354,20 +361,20 @@ class Alarm(base.Base):
         return cls(alarm_id=None,
                    name="SwiftObjectAlarm",
                    description="An alarm",
-                   type='combination',
+                   type='gnocchi_aggregation_by_metrics_threshold',
                    time_constraints=[AlarmTimeConstraint.sample().as_dict()],
                    user_id="c96c887c216949acbdfbd8b494863567",
                    project_id="c96c887c216949acbdfbd8b494863567",
                    enabled=True,
-                   timestamp=datetime.datetime.utcnow(),
+                   timestamp=datetime.datetime(2015, 1, 1, 12, 0, 0, 0),
                    state="ok",
                    severity="moderate",
-                   state_timestamp=datetime.datetime.utcnow(),
+                   state_reason="threshold over 90%",
+                   state_timestamp=datetime.datetime(2015, 1, 1, 12, 0, 0, 0),
                    ok_actions=["http://site:8000/ok"],
                    alarm_actions=["http://site:8000/alarm"],
                    insufficient_data_actions=["http://site:8000/nodata"],
                    repeat_actions=False,
-                   combination_rule=combination.AlarmCombinationRule.sample(),
                    )
 
     def as_dict(self, db_model):
@@ -375,7 +382,8 @@ class Alarm(base.Base):
         for k in d:
             if k.endswith('_rule'):
                 del d[k]
-        d['rule'] = getattr(self, "%s_rule" % self.type).as_dict()
+        rule = getattr(self, "%s_rule" % self.type)
+        d['rule'] = rule if isinstance(rule, dict) else rule.as_dict()
         if self.time_constraints:
             d['time_constraints'] = [tc.as_dict()
                                      for tc in self.time_constraints]
@@ -383,7 +391,28 @@ class Alarm(base.Base):
 
     @staticmethod
     def _is_trust_url(url):
-        return url.scheme in ('trust+http', 'trust+https')
+        return url.scheme.startswith('trust+')
+
+    @staticmethod
+    def _scrub_action_url(action):
+        """Remove trust ID from a URL."""
+        url = netutils.urlsplit(action)
+        if Alarm._is_trust_url(url):
+            netloc = url.netloc.rsplit('@', 1)[-1]
+            url = urlparse.SplitResult(url.scheme, netloc,
+                                       url.path, url.query,
+                                       url.fragment)
+        return url.geturl()
+
+    def _get_existing_trust_ids(self):
+        for action in itertools.chain(self.ok_actions or [],
+                                      self.alarm_actions or [],
+                                      self.insufficient_data_actions or []):
+            url = netutils.urlsplit(action)
+            if self._is_trust_url(url):
+                trust_id = url.username
+                if trust_id and url.password == 'delete':
+                    yield trust_id
 
     def update_actions(self, old_alarm=None):
         trustor_user_id = pecan.request.headers.get('X-User-Id')
@@ -394,47 +423,49 @@ class Alarm(base.Base):
         else:
             roles = []
         auth_plugin = pecan.request.environ.get('keystone.token_auth')
+
+        if old_alarm:
+            prev_trust_ids = set(old_alarm._get_existing_trust_ids())
+        else:
+            prev_trust_ids = set()
+        trust_id = prev_trust_ids.pop() if prev_trust_ids else None
+        trust_id_used = False
+
         for actions in (self.ok_actions, self.alarm_actions,
                         self.insufficient_data_actions):
             if actions is not None:
                 for index, action in enumerate(actions[:]):
                     url = netutils.urlsplit(action)
                     if self._is_trust_url(url):
-                        if '@' not in url.netloc:
+                        if '@' in url.netloc:
+                            errmsg = _("trust URL cannot contain a trust ID.")
+                            raise base.ClientSideError(errmsg)
+                        if trust_id is None:
                             # We have a trust action without a trust ID,
                             # create it
                             trust_id = keystone_client.create_trust_id(
                                 pecan.request.cfg,
                                 trustor_user_id, trustor_project_id, roles,
                                 auth_plugin)
-                            netloc = '%s:delete@%s' % (trust_id, url.netloc)
-                            url = list(url)
-                            url[1] = netloc
-                            actions[index] = urlparse.urlunsplit(url)
-        if old_alarm:
-            new_actions = list(itertools.chain(
-                self.ok_actions or [],
-                self.alarm_actions or [],
-                self.insufficient_data_actions or []))
-            for action in itertools.chain(
-                    old_alarm.ok_actions or [],
-                    old_alarm.alarm_actions or [],
-                    old_alarm.insufficient_data_actions or []):
-                if action not in new_actions:
-                    self.delete_trust(action)
+                        if trust_id_used:
+                            pw = ''
+                        else:
+                            pw = ':delete'
+                            trust_id_used = True
+                        netloc = '%s%s@%s' % (trust_id, pw, url.netloc)
+                        url = urlparse.SplitResult(url.scheme, netloc,
+                                                   url.path, url.query,
+                                                   url.fragment)
+                        actions[index] = url.geturl()
+        if trust_id is not None and not trust_id_used:
+            prev_trust_ids.add(trust_id)
+        for old_trust_id in prev_trust_ids:
+            keystone_client.delete_trust_id(old_trust_id, auth_plugin)
 
     def delete_actions(self):
-        for action in itertools.chain(self.ok_actions or [],
-                                      self.alarm_actions or [],
-                                      self.insufficient_data_actions or []):
-            self.delete_trust(action)
-
-    def delete_trust(self, action):
         auth_plugin = pecan.request.environ.get('keystone.token_auth')
-        url = netutils.urlsplit(action)
-        if self._is_trust_url(url) and url.password:
-            keystone_client.delete_trust_id(pecan.request.cfg,
-                                            url.username, auth_plugin)
+        for trust_id in self._get_existing_trust_ids():
+            keystone_client.delete_trust_id(trust_id, auth_plugin)
 
 
 Alarm.add_attributes(**{"%s_rule" % ext.name: ext.plugin
@@ -480,7 +511,7 @@ class AlarmChange(base.Base):
                    user_id="3e5d11fda79448ac99ccefb20be187ca",
                    project_id="b6f16144010811e387e4de429e99ee8c",
                    on_behalf_of="92159030020611e3b26dde429e99ee8c",
-                   timestamp=datetime.datetime.utcnow(),
+                   timestamp=datetime.datetime(2015, 1, 1, 12, 0, 0, 0),
                    )
 
 
@@ -491,7 +522,14 @@ def _send_notification(event, payload):
     notifier = messaging.get_notifier(transport, publisher_id="aodh.api")
     # FIXME(sileht): perhaps we need to copy some infos from the
     # pecan request headers like nova does
-    notifier.info(context.RequestContext(), notification, payload)
+    notifier.info({}, notification, payload)
+
+
+def stringify_timestamps(data):
+    """Stringify any datetimes in given dict."""
+    return dict((k, v.isoformat()
+                 if isinstance(v, datetime.datetime) else v)
+                for (k, v) in six.iteritems(data))
 
 
 class AlarmController(rest.RestController):
@@ -506,27 +544,35 @@ class AlarmController(rest.RestController):
         pecan.request.context['alarm_id'] = alarm_id
         self._id = alarm_id
 
-    def _alarm(self):
-        self.conn = pecan.request.alarm_storage_conn
+    def _enforce_rbac(self, rbac_directive):
+        # TODO(sileht): We should be able to relax this since we
+        # pass the alarm object to the enforcer.
         auth_project = rbac.get_limited_to_project(pecan.request.headers,
                                                    pecan.request.enforcer)
-        alarms = list(self.conn.get_alarms(alarm_id=self._id,
-                                           project=auth_project))
+        alarms = list(pecan.request.storage.get_alarms(alarm_id=self._id,
+                                                       project=auth_project))
         if not alarms:
             raise base.AlarmNotFound(alarm=self._id, auth_project=auth_project)
-        return alarms[0]
+        alarm = alarms[0]
+        target = {'user_id': alarm.user_id,
+                  'project_id': alarm.project_id}
+        rbac.enforce(rbac_directive, pecan.request.headers,
+                     pecan.request.enforcer, target)
+        return alarm
 
     def _record_change(self, data, now, on_behalf_of=None, type=None):
         if not pecan.request.cfg.record_history:
             return
+        if not data:
+            return
         type = type or models.AlarmChange.RULE_CHANGE
-        scrubbed_data = utils.stringify_timestamps(data)
+        scrubbed_data = stringify_timestamps(data)
         detail = json.dumps(scrubbed_data)
         user_id = pecan.request.headers.get('X-User-Id')
         project_id = pecan.request.headers.get('X-Project-Id')
         on_behalf_of = on_behalf_of or project_id
         severity = scrubbed_data.get('severity')
-        payload = dict(event_id=str(uuid.uuid4()),
+        payload = dict(event_id=uuidutils.generate_uuid(),
                        alarm_id=self._id,
                        type=type,
                        detail=detail,
@@ -537,7 +583,7 @@ class AlarmController(rest.RestController):
                        severity=severity)
 
         try:
-            self.conn.record_alarm_change(payload)
+            pecan.request.storage.record_alarm_change(payload)
         except aodh.NotImplementedError:
             pass
 
@@ -545,14 +591,30 @@ class AlarmController(rest.RestController):
         payload['detail'] = scrubbed_data
         _send_notification(type, payload)
 
+    def _record_delete(self, alarm):
+        if not alarm:
+            return
+        type = models.AlarmChange.DELETION
+        detail = {'state': alarm.state}
+        user_id = pecan.request.headers.get('X-User-Id')
+        project_id = pecan.request.headers.get('X-Project-Id')
+        payload = dict(event_id=uuidutils.generate_uuid(),
+                       alarm_id=self._id,
+                       type=type,
+                       detail=detail,
+                       user_id=user_id,
+                       project_id=project_id,
+                       on_behalf_of=project_id,
+                       timestamp=timeutils.utcnow(),
+                       severity=alarm.severity)
+
+        pecan.request.storage.delete_alarm(alarm.alarm_id)
+        _send_notification(type, payload)
+
     @wsme_pecan.wsexpose(Alarm)
     def get(self):
         """Return this alarm."""
-
-        rbac.enforce('get_alarm', pecan.request.headers,
-                     pecan.request.enforcer)
-
-        return Alarm.from_db_model(self._alarm())
+        return Alarm.from_db_model_scrubbed(self._enforce_rbac('get_alarm'))
 
     @wsme_pecan.wsexpose(Alarm, body=Alarm)
     def put(self, data):
@@ -561,11 +623,8 @@ class AlarmController(rest.RestController):
         :param data: an alarm within the request body.
         """
 
-        rbac.enforce('change_alarm', pecan.request.headers,
-                     pecan.request.enforcer)
-
         # Ensure alarm exists
-        alarm_in = self._alarm()
+        alarm_in = self._enforce_rbac('change_alarm')
 
         now = timeutils.utcnow()
 
@@ -584,17 +643,10 @@ class AlarmController(rest.RestController):
         data.timestamp = now
         if alarm_in.state != data.state:
             data.state_timestamp = now
+            data.state_reason = ALARM_REASON_MANUAL
         else:
             data.state_timestamp = alarm_in.state_timestamp
-
-        # make sure alarms are unique by name per project.
-        if alarm_in.name != data.name:
-            alarms = list(self.conn.get_alarms(name=data.name,
-                                               project=data.project_id))
-            if alarms:
-                raise base.ClientSideError(
-                    _("Alarm with name=%s exists") % data.name,
-                    status_code=409)
+            data.state_reason = alarm_in.state_reason
 
         ALARMS_RULES[data.type].plugin.update_hook(data)
 
@@ -605,39 +657,39 @@ class AlarmController(rest.RestController):
         try:
             alarm_in = models.Alarm(**updated_alarm)
         except Exception:
-            LOG.exception(_("Error while putting alarm: %s") % updated_alarm)
+            LOG.exception("Error while putting alarm: %s", updated_alarm)
             raise base.ClientSideError(_("Alarm incorrect"))
 
-        alarm = self.conn.update_alarm(alarm_in)
+        alarm = pecan.request.storage.update_alarm(alarm_in)
 
         change = dict((k, v) for k, v in updated_alarm.items()
                       if v != old_alarm[k] and k not in
                       ['timestamp', 'state_timestamp'])
         self._record_change(change, now, on_behalf_of=alarm.project_id)
-        return Alarm.from_db_model(alarm)
+        return Alarm.from_db_model_scrubbed(alarm)
 
     @wsme_pecan.wsexpose(None, status_code=204)
     def delete(self):
         """Delete this alarm."""
 
-        rbac.enforce('delete_alarm', pecan.request.headers,
-                     pecan.request.enforcer)
-
         # ensure alarm exists before deleting
-        alarm = self._alarm()
-        self.conn.delete_alarm(alarm.alarm_id)
+        alarm = self._enforce_rbac('delete_alarm')
+        self._record_delete(alarm)
         alarm_object = Alarm.from_db_model(alarm)
         alarm_object.delete_actions()
 
-    @wsme_pecan.wsexpose([AlarmChange], [base.Query])
-    def history(self, q=None):
+    @wsme_pecan.wsexpose([AlarmChange], [base.Query], [str], int, str)
+    def history(self, q=None, sort=None, limit=None, marker=None):
         """Assembles the alarm history requested.
 
         :param q: Filter rules for the changes to be described.
+        :param sort: A list of pairs of sort key and sort dir.
+        :param limit: The maximum number of items to be return.
+        :param marker: The pagination query marker.
         """
 
-        rbac.enforce('alarm_history', pecan.request.headers,
-                     pecan.request.enforcer)
+        # Ensure alarm exists
+        self._enforce_rbac('alarm_history')
 
         q = q or []
         # allow history to be returned for deleted alarms, but scope changes
@@ -645,9 +697,12 @@ class AlarmController(rest.RestController):
         # avoid inappropriate cross-tenant visibility of alarm history
         auth_project = rbac.get_limited_to_project(pecan.request.headers,
                                                    pecan.request.enforcer)
-        conn = pecan.request.alarm_storage_conn
+        conn = pecan.request.storage
         kwargs = v2_utils.query_to_kwargs(
             q, conn.get_alarm_changes, ['on_behalf_of', 'alarm_id'])
+        if sort or limit or marker:
+            kwargs['pagination'] = v2_utils.get_pagination_options(
+                sort, limit, marker, models.AlarmChange)
         return [AlarmChange.from_db_model(ac)
                 for ac in conn.get_alarm_changes(self._id, auth_project,
                                                  **kwargs)]
@@ -660,19 +715,19 @@ class AlarmController(rest.RestController):
         :param state: an alarm state within the request body.
         """
 
-        rbac.enforce('change_alarm_state', pecan.request.headers,
-                     pecan.request.enforcer)
+        alarm = self._enforce_rbac('change_alarm_state')
 
         # note(sileht): body are not validated by wsme
         # Workaround for https://bugs.launchpad.net/wsme/+bug/1227229
         if state not in state_kind:
             raise base.ClientSideError(_("state invalid"))
         now = timeutils.utcnow()
-        alarm = self._alarm()
         alarm.state = state
         alarm.state_timestamp = now
-        alarm = self.conn.update_alarm(alarm)
-        change = {'state': alarm.state}
+        alarm.state_reason = ALARM_REASON_MANUAL
+        alarm = pecan.request.storage.update_alarm(alarm)
+        change = {'state': alarm.state,
+                  'state_reason': alarm.state_reason}
         self._record_change(change, now, on_behalf_of=alarm.project_id,
                             type=models.AlarmChange.STATE_TRANSITION)
         return alarm.state
@@ -680,12 +735,7 @@ class AlarmController(rest.RestController):
     @wsme_pecan.wsexpose(state_kind_enum)
     def get_state(self):
         """Get the state of this alarm."""
-
-        rbac.enforce('get_alarm_state', pecan.request.headers,
-                     pecan.request.enforcer)
-
-        alarm = self._alarm()
-        return alarm.state
+        return self._enforce_rbac('get_alarm_state').state
 
 
 class AlarmsController(rest.RestController):
@@ -700,12 +750,12 @@ class AlarmsController(rest.RestController):
         if not pecan.request.cfg.record_history:
             return
         type = models.AlarmChange.CREATION
-        scrubbed_data = utils.stringify_timestamps(data)
+        scrubbed_data = stringify_timestamps(data)
         detail = json.dumps(scrubbed_data)
         user_id = pecan.request.headers.get('X-User-Id')
         project_id = pecan.request.headers.get('X-Project-Id')
         severity = scrubbed_data.get('severity')
-        payload = dict(event_id=str(uuid.uuid4()),
+        payload = dict(event_id=uuidutils.generate_uuid(),
                        alarm_id=alarm_id,
                        type=type,
                        detail=detail,
@@ -731,12 +781,12 @@ class AlarmsController(rest.RestController):
         :param data: an alarm within the request body.
         """
         rbac.enforce('create_alarm', pecan.request.headers,
-                     pecan.request.enforcer)
+                     pecan.request.enforcer, {})
 
-        conn = pecan.request.alarm_storage_conn
+        conn = pecan.request.storage
         now = timeutils.utcnow()
 
-        data.alarm_id = str(uuid.uuid4())
+        data.alarm_id = uuidutils.generate_uuid()
         user_limit, project_limit = rbac.get_limited_to(pecan.request.headers,
                                                         pecan.request.enforcer)
 
@@ -762,45 +812,46 @@ class AlarmsController(rest.RestController):
 
         data.timestamp = now
         data.state_timestamp = now
+        data.state_reason = ALARM_REASON_DEFAULT
 
         ALARMS_RULES[data.type].plugin.create_hook(data)
 
         change = data.as_dict(models.Alarm)
 
         data.update_actions()
-        # make sure alarms are unique by name per project.
-        alarms = list(conn.get_alarms(name=data.name,
-                                      project=data.project_id))
-        if alarms:
-            raise base.ClientSideError(
-                _("Alarm with name='%s' exists") % data.name,
-                status_code=409)
 
         try:
             alarm_in = models.Alarm(**change)
         except Exception:
-            LOG.exception(_("Error while posting alarm: %s") % change)
+            LOG.exception("Error while posting alarm: %s", change)
             raise base.ClientSideError(_("Alarm incorrect"))
 
         alarm = conn.create_alarm(alarm_in)
         self._record_creation(conn, change, alarm.alarm_id, now)
-        v2_utils.set_resp_location_hdr("/v2/alarms/" + alarm.alarm_id)
-        return Alarm.from_db_model(alarm)
+        v2_utils.set_resp_location_hdr("/alarms/" + alarm.alarm_id)
+        return Alarm.from_db_model_scrubbed(alarm)
 
-    @wsme_pecan.wsexpose([Alarm], [base.Query])
-    def get_all(self, q=None):
+    @wsme_pecan.wsexpose([Alarm], [base.Query], [str], int, str)
+    def get_all(self, q=None, sort=None, limit=None, marker=None):
         """Return all alarms, based on the query provided.
 
         :param q: Filter rules for the alarms to be returned.
+        :param sort: A list of pairs of sort key and sort dir.
+        :param limit: The maximum number of items to be return.
+        :param marker: The pagination query marker.
         """
-
+        target = rbac.target_from_segregation_rule(
+            pecan.request.headers, pecan.request.enforcer)
         rbac.enforce('get_alarms', pecan.request.headers,
-                     pecan.request.enforcer)
+                     pecan.request.enforcer, target)
 
         q = q or []
         # Timestamp is not supported field for Simple Alarm queries
         kwargs = v2_utils.query_to_kwargs(
-            q, pecan.request.alarm_storage_conn.get_alarms,
+            q, pecan.request.storage.get_alarms,
             allow_timestamps=False)
-        return [Alarm.from_db_model(m)
-                for m in pecan.request.alarm_storage_conn.get_alarms(**kwargs)]
+        if sort or limit or marker:
+            kwargs['pagination'] = v2_utils.get_pagination_options(
+                sort, limit, marker, models.Alarm)
+        return [Alarm.from_db_model_scrubbed(m)
+                for m in pecan.request.storage.get_alarms(**kwargs)]
