@@ -12,18 +12,17 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
+import json
 import operator
 
 from oslo_config import cfg
 from oslo_log import log
-from oslo_serialization import jsonutils
 from oslo_utils import fnmatch
 from oslo_utils import timeutils
 import six
 
 from aodh import evaluator
-from aodh.i18n import _, _LE, _LW
+from aodh.i18n import _
 
 LOG = log.getLogger(__name__)
 
@@ -76,16 +75,16 @@ class Event(object):
         """Validate received event has mandatory parameters."""
 
         if not self.obj:
-            LOG.error(_LE('Received invalid event (empty or None)'))
+            LOG.error('Received invalid event (empty or None)')
             raise InvalidEvent()
 
         if not self.obj.get('event_type'):
-            LOG.error(_LE('Failed to extract event_type from event = %s'),
+            LOG.error('Failed to extract event_type from event = %s',
                       self.obj)
             raise InvalidEvent()
 
         if not self.obj.get('message_id'):
-            LOG.error(_LE('Failed to extract message_id from event = %s'),
+            LOG.error('Failed to extract message_id from event = %s',
                       self.obj)
             raise InvalidEvent()
 
@@ -168,8 +167,8 @@ class EventAlarmEvaluator(evaluator.Evaluator):
             try:
                 event = Event(e)
             except InvalidEvent:
-                LOG.warning(_LW('Event <%s> is invalid, aborting evaluation '
-                                'for it.'), e)
+                LOG.warning('Event <%s> is invalid, aborting evaluation '
+                            'for it.', e)
                 continue
 
             for id, alarm in six.iteritems(
@@ -177,8 +176,8 @@ class EventAlarmEvaluator(evaluator.Evaluator):
                 try:
                     self._evaluate_alarm(alarm, event)
                 except Exception:
-                    LOG.exception(_LE('Failed to evaluate alarm (id=%(a)s) '
-                                      'triggered by event = %(e)s.'),
+                    LOG.exception('Failed to evaluate alarm (id=%(a)s) '
+                                  'triggered by event = %(e)s.',
                                   {'a': id, 'e': e})
 
         LOG.debug('Finished event alarm evaluation.')
@@ -253,8 +252,8 @@ class EventAlarmEvaluator(evaluator.Evaluator):
                     'query <query=%(alarm_query)s>.') %
                   {'id': event.id,
                    'event_type': event.get_value('event_type'),
-                   'alarm_query': jsonutils.dumps(alarm.obj.rule['query'],
-                                                  sort_keys=True)})
+                   'alarm_query': json.dumps(alarm.obj.rule['query'],
+                                             sort_keys=True)})
         reason_data = {'type': 'event', 'event': event.obj}
         always_record = alarm.obj.repeat_actions
         self._refresh(alarm.obj, state, reason, reason_data, always_record)
