@@ -20,7 +20,6 @@ import os.path
 from alembic import command
 from alembic import config
 from alembic import migration
-from oslo_db import exception
 from oslo_db.sqlalchemy import session as db_session
 from oslo_db.sqlalchemy import utils as oslo_sql_utils
 from oslo_log import log
@@ -28,10 +27,11 @@ from oslo_utils import timeutils
 import six
 from sqlalchemy import asc
 from sqlalchemy import desc
+from sqlalchemy.engine import url as sqlalchemy_url
 from sqlalchemy import func
 from sqlalchemy.orm import exc
 
-from aodh.i18n import _LI
+import aodh
 from aodh import storage
 from aodh.storage import base
 from aodh.storage import models as alarm_api_models
@@ -72,9 +72,19 @@ class Connection(base.Connection):
         # oslo.db doesn't support options defined by Aodh
         for opt in storage.OPTS:
             options.pop(opt.name, None)
-        self._engine_facade = db_session.EngineFacade(url, **options)
+        self._engine_facade = db_session.EngineFacade(self.dress_url(url),
+                                                      **options)
         self.conf = conf
 
+    @staticmethod
+    def dress_url(url):
+        # If no explicit driver has been set, we default to pymysql
+        if url.startswith("mysql://"):
+            url = sqlalchemy_url.make_url(url)
+            url.drivername = "mysql+pymysql"
+            return str(url)
+        return url
+
     def disconnect(self):
         self._engine_facade.get_engine().dispose()
 
@@ -82,7 +92,7 @@ class Connection(base.Connection):
         cfg = config.Config(
             "%s/sqlalchemy/alembic/alembic.ini" % os.path.dirname(__file__))
         cfg.set_main_option('sqlalchemy.url',
-                            self.conf.database.connection)
+                            self.conf.database.connection.replace("%", "%%"))
         return cfg
 
     def upgrade(self, nocreate=False):
@@ -95,15 +105,8 @@ class Connection(base.Connection):
             ctxt = migration.MigrationContext.configure(engine.connect())
             current_version = ctxt.get_current_revision()
             if current_version is None:
-                try:
-                    models.Base.metadata.create_all(engine, checkfirst=False)
-                except exception.DBError:
-                    # Assume tables exist from Ceilometer, take control
-                    # FIXME(jd) Remove in Ocata
-                    command.stamp(cfg, "12fe8fac9fe4")
-                    command.upgrade(cfg, "head")
-                else:
-                    command.stamp(cfg, "head")
+                models.Base.metadata.create_all(engine, checkfirst=False)
+                command.stamp(cfg, "head")
             else:
                 command.upgrade(cfg, "head")
 
@@ -144,6 +147,7 @@ class Connection(base.Connection):
                                       project_id=row.project_id,
                                       state=row.state,
                                       state_timestamp=row.state_timestamp,
+                                      state_reason=row.state_reason,
                                       ok_actions=row.ok_actions,
                                       alarm_actions=row.alarm_actions,
                                       insufficient_data_actions=(
@@ -157,7 +161,7 @@ class Connection(base.Connection):
         return (self._row_to_alarm_model(x) for x in query.all())
 
     @staticmethod
-    def _get_pagination_query(query, pagination, api_model, model):
+    def _get_pagination_query(session, query, pagination, api_model, model):
         if not pagination.get('sort'):
             pagination['sort'] = api_model.DEFAULT_SORT
         marker = None
@@ -176,6 +180,9 @@ class Connection(base.Connection):
         # order when "severity" specified in sorts.
         for sort_key, sort_dir in pagination['sort'][::-1]:
             if sort_key == 'severity':
+                engine = session.connection()
+                if engine.dialect.name != "mysql":
+                    raise aodh.NotImplementedError
                 sort_dir_func = {'asc': asc, 'desc': desc}[sort_dir]
                 query = query.order_by(sort_dir_func(
                     func.field(getattr(model, sort_key), 'low',
@@ -230,7 +237,7 @@ class Connection(base.Connection):
                 query = query.filter(getattr(models.Alarm, key) != value)
 
         query = self._get_pagination_query(
-            query, pagination, alarm_api_models.Alarm, models.Alarm)
+            session, query, pagination, alarm_api_models.Alarm, models.Alarm)
         alarms = self._retrieve_alarms(query)
 
         # TODO(cmart): improve this by using sqlalchemy.func factory
@@ -290,7 +297,8 @@ class Connection(base.Connection):
                                             user_id=row.user_id,
                                             project_id=row.project_id,
                                             on_behalf_of=row.on_behalf_of,
-                                            timestamp=row.timestamp)
+                                            timestamp=row.timestamp,
+                                            severity=row.severity)
 
     def query_alarms(self, filter_expr=None, orderby=None, limit=None):
         """Yields a lists of alarms that match filter."""
@@ -368,7 +376,7 @@ class Connection(base.Connection):
                     models.AlarmChange.timestamp < end_timestamp)
 
         query = self._get_pagination_query(
-            query, pagination, alarm_api_models.AlarmChange,
+            session, query, pagination, alarm_api_models.AlarmChange,
             models.AlarmChange)
         return self._retrieve_alarm_history(query)
 
@@ -396,5 +404,5 @@ class Connection(base.Connection):
             deleted_rows = (session.query(models.AlarmChange)
                             .filter(models.AlarmChange.timestamp < valid_start)
                             .delete())
-            LOG.info(_LI("%d alarm histories are removed from database"),
+            LOG.info("%d alarm histories are removed from database",
                      deleted_rows)
