@@ -26,6 +26,11 @@ from aodh.tests import constants
 from aodh.tests.functional import db as tests_db
 
 
+ALARM_TYPE = 'gnocchi_aggregation_by_metrics_threshold'
+METRIC_IDS = ['41869681-5776-46d6-91ed-cccc43b6e4e3',
+              'a1fb80f4-c242-4f57-87c6-68f47521059e']
+
+
 class DBTestBase(tests_db.TestBase):
     @staticmethod
     def create_side_effect(method, exception_type, test_exception):
@@ -48,7 +53,7 @@ class AlarmTestBase(DBTestBase):
     def add_some_alarms(self):
         alarms = [alarm_models.Alarm(alarm_id='r3d',
                                      enabled=True,
-                                     type='threshold',
+                                     type=ALARM_TYPE,
                                      name='red-alert',
                                      description='my red-alert',
                                      timestamp=datetime.datetime(2015, 7,
@@ -56,6 +61,7 @@ class AlarmTestBase(DBTestBase):
                                      user_id='me',
                                      project_id='and-da-boys',
                                      state="insufficient data",
+                                     state_reason="insufficient data",
                                      state_timestamp=constants.MIN_DATETIME,
                                      ok_actions=[],
                                      alarm_actions=['http://nowhere/alarms'],
@@ -66,18 +72,15 @@ class AlarmTestBase(DBTestBase):
                                                             duration=300)],
                                      rule=dict(comparison_operator='eq',
                                                threshold=36,
-                                               statistic='count',
+                                               aggregation_method='count',
                                                evaluation_periods=1,
-                                               period=60,
-                                               meter_name='test.one',
-                                               query=[{'field': 'key',
-                                                       'op': 'eq',
-                                                       'value': 'value',
-                                                       'type': 'string'}]),
+                                               granularity=60,
+                                               metrics=METRIC_IDS),
+                                     severity='low'
                                      ),
                   alarm_models.Alarm(alarm_id='0r4ng3',
                                      enabled=True,
-                                     type='threshold',
+                                     type=ALARM_TYPE,
                                      name='orange-alert',
                                      description='a orange',
                                      timestamp=datetime.datetime(2015, 7,
@@ -85,6 +88,7 @@ class AlarmTestBase(DBTestBase):
                                      user_id='me',
                                      project_id='and-da-boys',
                                      state="insufficient data",
+                                     state_reason="insufficient data",
                                      state_timestamp=constants.MIN_DATETIME,
                                      ok_actions=[],
                                      alarm_actions=['http://nowhere/alarms'],
@@ -93,18 +97,15 @@ class AlarmTestBase(DBTestBase):
                                      time_constraints=[],
                                      rule=dict(comparison_operator='gt',
                                                threshold=75,
-                                               statistic='avg',
+                                               aggregation_method='avg',
                                                evaluation_periods=1,
-                                               period=60,
-                                               meter_name='test.forty',
-                                               query=[{'field': 'key2',
-                                                       'op': 'eq',
-                                                       'value': 'value2',
-                                                       'type': 'string'}]),
+                                               granularity=60,
+                                               metrics=METRIC_IDS),
+                                     severity='low'
                                      ),
                   alarm_models.Alarm(alarm_id='y3ll0w',
                                      enabled=False,
-                                     type='threshold',
+                                     type=ALARM_TYPE,
                                      name='yellow-alert',
                                      description='yellow',
                                      timestamp=datetime.datetime(2015, 7,
@@ -112,6 +113,7 @@ class AlarmTestBase(DBTestBase):
                                      user_id='me',
                                      project_id='and-da-boys',
                                      state="insufficient data",
+                                     state_reason="insufficient data",
                                      state_timestamp=constants.MIN_DATETIME,
                                      ok_actions=[],
                                      alarm_actions=['http://nowhere/alarms'],
@@ -120,19 +122,11 @@ class AlarmTestBase(DBTestBase):
                                      time_constraints=[],
                                      rule=dict(comparison_operator='lt',
                                                threshold=10,
-                                               statistic='min',
+                                               aggregation_method='min',
                                                evaluation_periods=1,
-                                               period=60,
-                                               meter_name='test.five',
-                                               query=[{'field': 'key2',
-                                                       'op': 'eq',
-                                                       'value': 'value2',
-                                                       'type': 'string'},
-                                                      {'field':
-                                                       'user_metadata.key3',
-                                                       'op': 'eq',
-                                                       'value': 'value3',
-                                                       'type': 'string'}]),
+                                               granularity=60,
+                                               metrics=METRIC_IDS),
+                                     severity='low'
                                      )]
 
         for a in alarms:
@@ -172,10 +166,8 @@ class AlarmTest(AlarmTestBase):
 
     def test_list_by_type(self):
         self.add_some_alarms()
-        alarms = list(self.alarm_conn.get_alarms(alarm_type='threshold'))
+        alarms = list(self.alarm_conn.get_alarms(alarm_type=ALARM_TYPE))
         self.assertEqual(3, len(alarms))
-        alarms = list(self.alarm_conn.get_alarms(alarm_type='combination'))
-        self.assertEqual(0, len(alarms))
 
     def test_list_excluded_by_name(self):
         self.add_some_alarms()
@@ -190,37 +182,34 @@ class AlarmTest(AlarmTestBase):
         alarms = list(self.alarm_conn.get_alarms())
         self.assertEqual(3, len(alarms))
 
-        meter_names = sorted([a.rule['meter_name'] for a in alarms])
-        self.assertEqual(['test.five', 'test.forty', 'test.one'], meter_names)
+        metrics = sorted([a.rule['metrics'] for a in alarms])
+        self.assertEqual([METRIC_IDS, METRIC_IDS, METRIC_IDS], metrics)
 
     def test_update(self):
         self.add_some_alarms()
+        metrics = ['6841c175-d7c4-4bc2-bc7a-1c7832271b8f',
+                   'bc1efaa5-93b4-4518-8337-18519917c15a']
         orange = list(self.alarm_conn.get_alarms(name='orange-alert'))[0]
         orange.enabled = False
         orange.state = alarm_models.Alarm.ALARM_INSUFFICIENT_DATA
-        query = [{'field': 'metadata.group',
-                  'op': 'eq',
-                  'value': 'test.updated',
-                  'type': 'string'}]
-        orange.rule['query'] = query
-        orange.rule['meter_name'] = 'new_meter_name'
+        orange.rule['metrics'] = metrics
         updated = self.alarm_conn.update_alarm(orange)
         self.assertFalse(updated.enabled)
         self.assertEqual(alarm_models.Alarm.ALARM_INSUFFICIENT_DATA,
                          updated.state)
-        self.assertEqual(query, updated.rule['query'])
-        self.assertEqual('new_meter_name', updated.rule['meter_name'])
+        self.assertEqual(metrics, updated.rule['metrics'])
 
     def test_update_llu(self):
         llu = alarm_models.Alarm(alarm_id='llu',
                                  enabled=True,
-                                 type='threshold',
+                                 type=ALARM_TYPE,
                                  name='llu',
                                  description='llu',
                                  timestamp=constants.MIN_DATETIME,
                                  user_id='bla',
                                  project_id='ffo',
                                  state="insufficient data",
+                                 state_reason="insufficient data",
                                  state_timestamp=constants.MIN_DATETIME,
                                  ok_actions=[],
                                  alarm_actions=[],
@@ -229,11 +218,10 @@ class AlarmTest(AlarmTestBase):
                                  time_constraints=[],
                                  rule=dict(comparison_operator='lt',
                                            threshold=34,
-                                           statistic='max',
+                                           aggregation_method='max',
                                            evaluation_periods=1,
-                                           period=60,
-                                           meter_name='llt',
-                                           query=[])
+                                           granularity=60,
+                                           metrics=METRIC_IDS)
                                  )
         updated = self.alarm_conn.create_alarm(llu)
         updated.state = alarm_models.Alarm.ALARM_OK
@@ -265,7 +253,6 @@ class AlarmTest(AlarmTestBase):
             self.assertNotEqual(victim.name, s.name)
 
 
-@tests_db.run_with('sqlite', 'mysql', 'pgsql', 'hbase')
 class AlarmHistoryTest(AlarmTestBase):
 
     def setUp(self):
@@ -318,6 +305,27 @@ class AlarmHistoryTest(AlarmTestBase):
         history = list(self.alarm_conn.query_alarm_history())
         self.assertEqual(0, len(history))
 
+    def test_record_severity_when_alarm_change(self):
+        alarm = list(self.alarm_conn.get_alarms(name='orange-alert'))[0]
+        severity = "low"
+        alarm_change = {
+            "event_id": "3d22800c-a3ca-4991-b34b-d97efb6047d9",
+            "alarm_id": alarm.alarm_id,
+            "type": alarm_models.AlarmChange.STATE_TRANSITION,
+            "detail": "detail %s" % alarm.name,
+            "user_id": alarm.user_id,
+            "project_id": alarm.project_id,
+            "on_behalf_of": alarm.project_id,
+            "severity": severity,
+            "timestamp": datetime.datetime(2014, 4, 7, 7, 34)
+            }
+        self.alarm_conn.record_alarm_change(alarm_change=alarm_change)
+        filter_expr = {"=": {"severity": "low"}}
+        history = list(self.alarm_conn.query_alarm_history(
+            filter_expr=filter_expr))
+        self.assertEqual(1, len(history))
+        self.assertEqual("low", history[0].severity)
+
 
 class ComplexAlarmQueryTest(AlarmTestBase):
 
